#!/usr/bin/env python
"""
Simple script to build 'stable' ETS eggs

Copyright (c) 2007 by Enthought, Inc.
License: BSD Style.

"""

import glob
import optparse
import os
import shutil
import subprocess
import sys


def error(msg):
    bar = 78*'*'
    print '\n%s\nERROR: %s\n%s\n' % (bar, msg, bar)

    return


def find_component_dirs():
    d = []
    for file in os.listdir('.'):
        if os.path.isdir(file):
            if os.path.exists(os.path.join(file, 'setup.py')):
                d.append(file)

    d.sort()

    return d


def build_eggs(dirs, dest='dist', release=False, source=False, verbose=False):

    # Ensure the destination directory exists.
    dest = os.path.abspath(dest)
    if not os.path.exists(dest):
        os.mkdir(dest)

    # Create the build command to be run.
    args = [sys.executable, 'setup.py']
    if not verbose:
        args.extend(['--quiet'])
    if release:
        args.extend(['release'])
    if source:
        args.extend(['egg_info', 'sdist'])
    else:
        args.extend(['build'])
        if sys.platform=='win32':
            args.extend(['--compiler=mingw32'])
        args.extend(['bdist_egg'])
    args.extend(['-d', dest])

    # Build an egg or source tarball out of each specified directory.
    for d in dirs:
        print "=" * 78
        print 'Building directory: %s\n' % d

        # Make sure the component directory exists.
        if not os.path.isdir(d):
            error('Can not find directory "%s"' % d)
            continue

        # Make sure the setup.py exists.
        if not os.path.exists(os.path.join(d, 'setup.py')):
            error('Can not find setup.py')
            continue

        # Run the build command.
        print "Executing: %s" % (" ".join(args))
        ret = subprocess.call(args, cwd=d)
        if ret != 0:
            error('Build error: exited with value = %s' % ret)

        print ''

    # Generate the egg_info files for Enstaller to use when browsing the repo.
    if not source:
        try:
            from enthought.enstaller.egg import Egg

            print '\n\n' + '*'*78
            print 'Building egg_info files...'
            for file in glob.glob(os.path.join(dest, '*.egg')):
                print '  %s' % file
                egg = Egg(file)
                egg.create_info_file()
        except Exception, e:
            print 'WARNING: Could not build egg.info files: %s' % e

    return


def clean(args, dest='dist'):
    for d in args:
        print "Cleaning: %s" % d
        try:
            shutil.rmtree(os.path.join(d, 'build'))
        except OSError:
            pass
        try:
            shutil.rmtree(os.path.join(d, 'dist'))
        except OSError:
            pass
        try:
            shutil.rmtree(os.path.join(d, '%s.egg-info' % d))
        except OSError:
            pass

    print 'Cleaning: %s' % dest
    try:
        shutil.rmtree(dest)
    except OSError:
        pass

    return


def main():
    version = '%prog 0.1'
    usage = '%prog [options] [directories]'
    description = ('Build "stable" ETS component eggs.  If no explicit '
        'directories are specified, then will scan the current directory '
        'tree looking for component directories to build / clean.')

    parser = optparse.OptionParser(version=version, description=description,
        usage=usage)
    parser.add_option('-d', '--dest-dir', action='store', dest='dest',
        type='string', default='dist',
        help='specify the destination directory for built eggs ' \
        '(defaults to %default)')
    parser.add_option('-r', '--release', action='store_true', dest='release',
        help='build a release version')
    parser.add_option('-s', '--source', action='store_true', dest='source',
        help='build source tarballs as well as eggs, stored in the same '\
        'destination')
    parser.add_option('-v', '--verbose', action='store_true', dest='verbose',
        help='show all output from the build commands')
    parser.add_option('--clean', action='store_true', dest='clean',
        help='perform a clean instead of building anything')
    options, args = parser.parse_args()

    # If no explicit directory specification provided, use all of the component
    # directories we can find.
    print ''
    if len(args) < 1:
        print 'Finding component directories to operate on...'
        args = find_component_dirs()
        print '...Done\n'

    # Perform a clean if the user requested it, otherwise build the target eggs.
    if options.clean:
        clean(args, dest=options.dest)
    else:
        build_eggs(args, dest=options.dest, release=options.release,
            source=options.source, verbose=options.verbose)

    return


if __name__ == '__main__':
    main()

